/*
# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Santé)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------  
*/

/*=========================================================================
Program:   bbtk
Module:    $RCSfile$
Language:  C++
Date:      $Date$
Version:   $Revision$
=========================================================================*/

/* ---------------------------------------------------------------------

 * Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
 * Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
 *
 *  This software is governed by the CeCILL-B license under French law and
 *  abiding by the rules of distribution of free software. You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
 *  or in the file LICENSE.txt.
 *
 *  As a counterpart to the access to the source code and  rights to copy,
 *  modify and redistribute granted by the license, users are provided only
 *  with a limited warranty  and the software's author,  the holder of the
 *  economic rights,  and the successive licensors  have only  limited
 *  liability.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 * ------------------------------------------------------------------------ */


/****
 * Design and Developpement of BBTK GEditor
 * Ricardo A Corredor J <ra.corredor67@uniandes.edu.co>
 * RaC - 2010
 ****/

#ifndef __GlobalConstants_h__
#define __GlobalConstants_h__

namespace bbtk {

	// Object types
	const int GBLACKBOX					=	1;
	const int GCONNECTOR				=	2;
	const int GPORT						=	3;
	const int GINPUTPORT				=	4;
	const int GOUTPUTPORT				=	5;
	const int GCOMPLEXINPUTPORT			=	6;
	const int GCOMPLEXOUTPUTPORT		=	7;

	const double GPOSITION_Z			=	-900;
	
	// Object states
	
	// JGRR
	const int POSSIBLE_CONNECTION 		= 	107;
	const int SELECTED_POSSIBLE_CONNECTION = 108; 
	// EO JGRR
    
	const int NOTHING_HAPPENS			=	101;
	const int HIGHLIGHTED				=	102;
	const int CLICKED					=	103;
	const int DRAG						=	104;
	const int SELECTED					=	105;
	const int CREATING_CONTOUR			=	106;

	// Object dimensions
	const double BOX_HEIGHT				=	10;
	const double BOX_WIDTH				=	100;
	const double PORT_HEIGHT			=	1.8;
	const double PORT_WIDTH				=	2.8;
	const double COMPLEXPORT_HEIGHT		=	8;
	const double COMPLEXPORT_WIDTH		=	8;

	// Commands
	const int NO_COMMAND				=	201;
	const int INIT_CREATION_CONTOUR		=	202;
	const int FIN_CREATION_CONTOUR		=	203;
	const int ADD_TO_SELECTED			=	204;
	const int DRAG_OBJECTS				=	205;
	const int EDIT_BLACKBOX				=	206;
	const int REMOVE_FROM_SELECTED		=	207;


	//// COLORS (object_state_(R|G|B))

	//Object Border colors

	const double BOXBORDER_NH_R					=	0.5;
	const double BOXBORDER_NH_G					=	0.25;
	const double BOXBORDER_NH_B					=	0.0;

	const double COMPLEXINPUTPORTBORDER_NH_R	=	0.7;
	const double COMPLEXINPUTPORTBORDER_NH_G	=	0.1;
	const double COMPLEXINPUTPORTBORDER_NH_B	=	0.1;

	const double COMPLEXOUTPUTPORTBORDER_NH_R	=	0.3;
	const double COMPLEXOUTPUTPORTBORDER_NH_G	=	0.55;
	const double COMPLEXOUTPUTPORTBORDER_NH_B	=	0.1;

	const double BOXBORDER_HIGHLIGHTED_R		=	0.05;
	const double BOXBORDER_HIGHLIGHTED_G		=	0.25;
	const double BOXBORDER_HIGHLIGHTED_B		=	0.45;

	const double BOXBORDER_DRAG_R				=	0.75;
	const double BOXBORDER_DRAG_G				=	0.75;
	const double BOXBORDER_DRAG_B				=	0.75;

	const double BOXBORDER_SELECTED_R			=	0.45;
	const double BOXBORDER_SELECTED_G			=	0.45;
	const double BOXBORDER_SELECTED_B			=	0.00;


	//Object Fill colors

	const double BOXFILL_NH_R				=	0.5;
	const double BOXFILL_NH_G				=	0.35;
	const double BOXFILL_NH_B				=	0.15;

	const double COMPLEXINPUTPORTFILL_NH_R	=	0.8;
	const double COMPLEXINPUTPORTFILL_NH_G	=	0.1;
	const double COMPLEXINPUTPORTFILL_NH_B	=	0.1;

	const double COMPLEXOUTPUTPORTFILL_NH_R	=	0.5;
	const double COMPLEXOUTPUTPORTFILL_NH_G	=	0.85;
	const double COMPLEXOUTPUTPORTFILL_NH_B	=	0.1;

	const double BOXFILL_HIGHLIGHTED_R		=	0.05;
	const double BOXFILL_HIGHLIGHTED_G		=	0.30;
	const double BOXFILL_HIGHLIGHTED_B		=	0.45;

	const double BOXFILL_DRAG_R				=	0.75;
	const double BOXFILL_DRAG_G				=	0.75;
	const double BOXFILL_DRAG_B				=	0.75;

	const double BOXFILL_SELECTED_R			=	0.65;
	const double BOXFILL_SELECTED_G			=	0.65;
	const double BOXFILL_SELECTED_B			=	0.05;


	//Object Text colors


	const double PORTFILL_NH_R				=	0.0;
	const double PORTFILL_NH_G				=	0.0;
	const double PORTFILL_NH_B				=	0.0;

	const double PORTTEXT_NH_R				=	1.0;
	const double PORTTEXT_NH_G				=	1.0;
	const double PORTTEXT_NH_B				=	0.0;

	const double BOXTEXT_NH_R				=	0.25;
	const double BOXTEXT_NH_G				=	0.0;
	const double BOXTEXT_NH_B				=	0.0;

	const double BOXTEXT_HIGHLIGHTED_R		=	0.05;
	const double BOXTEXT_HIGHLIGHTED_G		=	0.05;
	const double BOXTEXT_HIGHLIGHTED_B		=	0.4;

	const double BOXTEXT_DRAG_R				=	0.75;
	const double BOXTEXT_DRAG_G				=	0.75;
	const double BOXTEXT_DRAG_B				=	0.75;

	const double BOXTEXT_SELECTED_R			=	0.35;
	const double BOXTEXT_SELECTED_G			=	0.35;
	const double BOXTEXT_SELECTED_B			=	0.05;

	// GUI Id's
	const int ID_NEW						=	1000;
	const int ID_SAVE_AS_DIAGRAM			=	1001;
	const int ID_SAVE_DIAGRAM				=	1015; //last DFCH
	const int ID_OPEN						=	1002;
	const int ID_RUN						=	1003;
	const int ID_DELETEALL					=	1004;
	const int ID_CENTERVIEW					=	1005;
	const int ID_BTNCOMPLEXBOX				=	1006;
	const int ID_ADDCOMPLEXBOXINPUT			=	1007;
	const int ID_ADDCOMPLEXBOXOUTPUT		=	1008;
	const int ID_OPEN_BBS					=	1009;
	const int ID_SAVE_AS_BBS				=	1010;
	const int ID_SAVE_AS_COMPLEXBOX			=	1011;
	const int ID_COPY_TO_COMPLEXDIAGRAM		=	1012;
	const int ID_BTNBOX						=	1013;
	const int ID_HELP_BBEDITOR				=	1014;
	const int ID_BTEXECUTABLEBOX			=	1016; // EED
	const int ID_EXPORT_CONSOLEAPP			=	1017; // RaC
	const int ID_EDIT_COMPLEXBOX_SCRIPT		=	1018; // EED
	
	const int wxID_NOTEBOOK					=	1018;  // EED updated
	
	const int ID_bbEditor_Tool_Menu_CreatePackage	=	1019;  // EED
	const int ID_bbEditor_Tool_Menu_CreateBlackBox		=	1020;  // EED
	const int ID_bbEditor_Tool_Menu_PlugPackage			=	1021;  // EED
	const int ID_bbEditor_Tool_Menu_EditConfig			=	1022;  // EED
	const int ID_bbEditor_Tool_Menu_ShowHTMLDoc			=	1023;  // EED
	const int ID_bbEditor_Tool_Menu_CreateIndex			=	1024;  // EED
	
	const int ID_UNDO									=	1025;  // FCY
	const int ID_REDO									=	1026;  // FCY
	const int ID_CHANGENAME								=   1027;

	const int ID_EDITPROPERTIES									=	1028;  // RaC 2nd HackFest
	const int ID_SHOWTREE									=	1029;  // CFT
}
// namespace bbtk
#endif

