/*
# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Santé)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------  
*/

/*=========================================================================
Program:   bbtk
Module:    $RCSfile$
Language:  C++
Date:      $Date$
Version:   $Revision$
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and
*  abiding by the rules of distribution of free software. You can  use,
*  modify and/ or redistribute the software under the terms of the CeCILL-B
*  license as circulated by CEA, CNRS and INRIA at the following URL
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability.
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */



/**
*  \file
*  \brief Class bbtk::BlackBox : abstract black-box interface.
*/

/****
* Design and Developpement of BBTK GEditor
* Ricardo A Corredor J <ra.corredor67@uniandes.edu.co>
* RaC - 2010
****/

#ifndef __wxGEditorTabPanel_h__
#define __wxGEditorTabPanel_h__

//Includes same project
#include "wxVtkSceneManager.h"
#include "wxTabPanelsManager.h"

//Includes wxWidgets
#include <wx/panel.h>
#include <wx/aui/aui.h>
#include <wx/dnd.h>

//Includes creaMaracasVisu
#include <wxVtk3DBaseView.h>

//Includes std
#include <fstream>

namespace bbtk
{

	class tmpClasswxTextDropTarget :   public wxTextDropTarget
	{
		public:
 		  virtual bool OnDropText(wxCoord x, wxCoord y, const wxString& data) { return false;}
	};


	//RaC: It is important if it exists a double relation
	//e.g. wxGEditorTabPanel includes wxVtkSceneManager, and wxVtkSceneManager includes wxGEditorTabPanel
	class wxVtkSceneManager;
	class wxTabPanelsManager;

	// RaC Be careful with the double heritance
	class wxGEditorTabPanel :  public wxPanel, public wxTextDropTarget
	{
	public:
		wxGEditorTabPanel();
		wxGEditorTabPanel(wxWindow *parent, int id, Factory::Pointer bbtkfactory);
		~wxGEditorTabPanel();

		void initWxVtkCanvas();

		// Sets the parent manager
		void setPanelsManager(wxTabPanelsManager* panelsManager);

		// Get the BBS script of the pipeline diagram included in the panel
		std::string getDiagramBBS(bool wln=false);

		// Saves the actual BBS as complex box
		// RaC TOFIX It must be included the package of the complex box
		std::string saveComplexBoxBBS();

		// Shows the dialog to edit black box parameters
		void editBlackBox(GBlackBoxModel *bbmodel);

		// Save diagram as BBG
		void saveDiagram(std::string &content, const std::string &path); //DFCH

		// Save temporary diagram as BBG
		void saveTempDiagram(const std::string &);

		// Save temporary diagram as BBG and update buttons
		void saveTempandUpdate(const std::string &);

		// Load BBG diagram
		void loadDiagram(ifstream &inputStream, const std::string &path); //DFCH

		void saveStringAction(const std::string &action);

		// Load temporary diagram as BBG
		void  loadTempDiagram(unsigned short un);

		// evaluate the position in the undo actions list
		unsigned short getUndoState(std::string &);

		// evaluate the position in the redo actions list
		unsigned short getRedoState(std::string &);

		// Add a new complex input object to the scene
		void addComplexInputPort(std::string portName);

		// Add a new complex output object to the scene
		void addComplexOutputPort(std::string portName);

		// Delete all boxes in the diagram
		void deleteAllBoxes();

		// Center the camera in the initial position
		void centerView();

		// Display feedback info in the GUI
		void displayBlackBoxInfo(std::string packageName, std::string boxName);
		void updateStatusBar(std::string textStatus);

		int getPanelId();
		bool isComplexBox();
		void setComplexBox(bool val);

		// Returns a map with the id,controller of all the objects selected
		std::map<int,GObjectController*> getSelectedObjects();

		void addObjects(std::map<int,GObjectController*> objectsMap);
		int getNumSelectedObjects();

		// Receives the string from a drag and drop source as for example the BBTK Package Browser
		virtual bool OnDropText(wxCoord x, wxCoord y, const wxString& data);

		wxVtkSceneManager* getSceneManager();

		void SetFullPath( const std::string& );
		void SetFileName( const std::string& );
		void SetCbName(std::string cbName);
        void SetCbPackageName(std::string packagename);
        void SetAuthor(std::string author);
        void SetCategory(std::string category);
        void SetDescription(std::string description);
		void SetMessageKind(std::string kind);
        void SetMessageLevel(std::string level);
        //Getters
        std::string GetFullPath( );
        std::string GetFileName( );
        std::string GetCbName();
        std::string GetCbPackageName();
        std::string GetAuthor();
        std::string GetDescription();
        std::string GetCategory();
		std::string GetMessageKind();
        std::string GetMessageLevel();

	//=========================================================================

	private:
		std::vector<std::string> states;				// represents each modification of the tab. Maxixum size 100!!
		std::vector<std::string>::iterator _actualdo;	// actual position on modifications vector
		std::vector<std::string> sstates;				// represents string of each modification of the tab. Maxixum size 100!!
		std::vector<std::string>::iterator _sactualdo;	// actual position onstring  modifications vector
		int						_id;
		wxAuiManager			*_panelAUIMgr;
		wxVtkSceneManager		*_sceneManager;
		wxTabPanelsManager		*_panelsManager;
		std::string				_fullPath;
		std::string				_fileName;
	protected:
		Factory::Pointer		_bbtkfactory;

	};


}
// namespace bbtk
#endif

